#! /usr/bin/perl
use strict;

my %conf;
my @regexes;
my @rules;
my @message;
my $msg_id = 0;
my %msg_ids;
my $inode;
my $now;
my $last;
my $title;

my $conf_file = ($ARGV[0]);

load_config();


#my $file = $conf{'log_file'};
my $pid_file = $conf{ 'pid_file' };

# write pid to pid_file
open (PIDFILE, ">$pid_file" ) || die "Could not open Pid File $pid_file \n";
print PIDFILE "$$\n";
close (PIDFILE);

# Setup our signal capture, so we can clean ourselves up should we die
#1 2 3 15
$SIG{INT}= \&catch_sig ;
$SIG{HUP}= \&catch_sig ;
$SIG{TERM}= \&catch_sig ;

my $title = "$conf{'log_file'}: message(s) of interest";
$title =~ s/^.*\///g;

my $sort_field = $conf{ 'sort_field' };

# open our log file
open_logfile($conf{'log_file'});

# seek to the end of the log file
if ( $conf{ 'jump_to_bottom_of_file' } eq "true" )
{
	seek(FILE,0,2);  
}


# now an infinite loop
for (;;)
{
	# read to end of logfile
	while (my $line = <FILE>)
	{
		my $matched = "NO";
		for ( my $i = 0; $i < scalar(@regexes) ; $i++ )
		{
			my $regex = $regexes[$i];
			if ( $line =~ qr/$regex/ )
			{
				if( $rules[$i] eq "exclude" )
				{
					$matched = "YES";
					$i = scalar(@regexes);
				}
				if( $rules[$i] eq "include" )
				{
					push(@message, $line);
					$matched = "YES";
					$i = scalar(@regexes);
				}
			}
		}
		if( $matched ne "YES" )
		{
			push(@message, $line);
		}
		
		# check if we need to send message
		check_msg_buffer();
	}

	# reset filehandle
	sleep 1;
	seek( FILE, 0 , 1 );

	# check if we need to send message
	check_msg_buffer();

}

sub open_logfile
{
	my($filename)=(@_);
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
    $atime,$mtime,$ctime,$blksize,$blocks)
            = stat($filename);
	if ( $ino != $inode )
	{
		if ( $conf{ 'debug' } eq "true" )
		{
			print "inode changed! new: \"$ino\" != old \"$inode\"\n";
		}
		close(FILE);
		open (FILE, "$filename" ) || print "Could not open Log File $filename \n";
		$inode = $ino;
	}
}

sub check_msg_buffer
{
	$now = time();
	if ( ($now - $last) < $conf{ 'msg_delay' } )
	{
		return;
	}

	if ( $conf{ 'debug' } eq "true" )
	{
		print "checking message buffer\n";
	}
	$last = time();
	load_config();
	open_logfile($conf{ 'log_file' });

	if ( scalar(@message) <= 0 )
	{
		return;
	}
	$msg_id++;
	if ( $conf{ 'debug' } eq "true" )
	{
		print "MESSAGE $msg_id " . scalar(@message) . " lines\n";
	}
	send_message(\@message);
	@message = ();
}

sub send_message
{
	my ($ra) = (@_);
	if ( $sort_field ne "" ) # this means sort the messages by $sort_field and send seperate messages for each value found in the $sort_field
	{
		my %msgs;
		foreach my $line (@$ra)
		{
			my (@junk) = split( " ", $line );
			my $src = $junk[$sort_field];
			push( @{$msgs{ $src }}, $line );
		}
		foreach my $src (keys %msgs)
		{
			$msg_ids{ $src }++;
			open( MAIL, "| /usr/bin/mail -s \"$title $msg_ids{ $src } from $src\" $conf{ 'email_recpt' }" );
			foreach my $line (@{$msgs{$src}})
			{
				print MAIL "$line";
			}
			close(MAIL);
			if ($conf{ 'debug' } eq "true" )
			{
				print "sent message: /usr/bin/mail -s \"$title $msg_ids{ $src } from $src\" $conf{ 'email_recpt' }\n";
			}
		}
	}
	else # This means send one big message
	{
		open( MAIL, "| /usr/bin/mail -s \"$title $msg_id\" $conf{ 'email_recpt' }" );
		foreach my $line (@{$ra})
		{
			print MAIL "$line";
		}
		close(MAIL);
		if ($conf{ 'debug' } eq "true" )
		{
			print "sent message: /usr/bin/mail -s \"$title $msg_id\" $conf{ 'email_recpt' }\n";
		}
	}
}

sub catch_sig
{
        print "$0: Caught Signal and exiting\n";
        close( FILE );
        unlink $pid_file;
        exit 0;
}

sub load_config
{
	if ( $conf{ 'debug' } eq "true" )
	{
		print "loading config $conf_file\n";
	}
	%conf = ();
	@regexes = ();
	@rules = ();
	open( CONF, $conf_file ) || die "could not open $conf_file for reading!\n";
	while (my $line = <CONF> )
	{
		chomp($line);
		if ( $line =~ /^#/ || $line =~ /^\w*$/ ) { next; }
		my ( $item, $value ) = split ( " ", $line, 2 );
		$conf{ $item } = $value;
		if ( $item eq "exclude" || $item eq "include" )
		{
			push(@regexes, $value);
			push(@rules, $item);
		}
	}
	close(CONF);
}
